#include "kdemowindow.moc"
#include <kapp.h>
#include <kstdaction.h>
#include <qiconset.h>
#include <qpixmap.h>
#include <kfiledialog.h>
#include <qdatetime.h>
#include <qtextstream.h>
#include <qfile.h>
#include <kdatastream.h>
#include <kmessagebox.h>
#include <ktoolbar.h>

KDemoWindow :: KDemoWindow() :
KMainWindow(0) // selbstndiges Fenster; Titeltext aus KApplication nehmen
{
	edit = new KEdit(this);
	setCentralWidget(edit); // View-Widget

	KStdAction::open(this, SLOT(openFile()), actionCollection());
	KStdAction::saveAs(this, SLOT(saveAsFile()), actionCollection());
	// eigenwillige Warnung zur Laufzeit: QToolBar::QToolBar main window cannot be 0.
	KStdAction::quit(KApplication::kApplication(), SLOT(closeAllWindows()), actionCollection());

	new KAction("&Lschen", QIconSet(QPixmap("clearicon.xpm")), CTRL+Key_L,
		edit, SLOT(clear()), actionCollection(), "clear");
	// sofort verdrahtet, Adresse brauchen wir hier nicht mehr
	// statt fester Pfade besser KStandardDirs verwenden
	new KAction("Da&tum einfgen", QIconSet(QPixmap("dateicon.xpm")), CTRL+Key_T,
		this, SLOT(insertDate()), actionCollection(), "date");
	wrapAction = new KToggleAction("Am &Fenster umbrechen", QIconSet(QPixmap("wrapicon.xpm")),
		CTRL+Key_F, this, SLOT(wrap()), actionCollection(), "wrap");
	// Toggle: an- und ausschalten
	
	KConfig* conf = KApplication::kApplication()->config();
	if(conf->readBoolEntry("Wrap", false))
	{
		edit->setWordWrap(KEdit::WidgetWidth);
		wrapAction->setChecked(true);
	}

	// Mens und Toolbars bauen
	createGUI(); // sucht lokal nach [Appname]ui.rc, wenn nicht ausdrcklich Pfad/Datei angegeben

	toolBar("mainToolBar")->setBarPos(KToolBar::Left);
}

KDemoWindow :: ~KDemoWindow()
{
}

void KDemoWindow :: openFile()
{
	QString filename = KFileDialog::getOpenFileName();
	if(filename.isEmpty()) // abgebrochen oder nichts gewhlt
	{
		return;
	}

	QFile f(filename);
	bool isOK = f.open(IO_ReadOnly); // IO_ReadOnly ist Makrokonstante, also ohne Namensraum
	if(!isOK)
	{
		KMessageBox::error(this, "Datei konnte nicht geffnet werden", "Dateifehler");
		return;
	}

	QTextStream s(&f); // fr Daten: QDataStream
	edit->clear(); // noch unschn gelst: bei Lesefehler auch alter Text kaputt
	edit->insertText(&s); // fr Daten z.B.: s >> i >> p;
	f.close(); // wrde auch vom Destruktor aufgerufen, aber dann knnten wir nicht schon hier Probleme abfragen
	
	if(f.status() != IO_Ok)
	{
		KMessageBox::error(this, "Fehler beim Lesen der Datei", "Dateifehler");
	}
}

void KDemoWindow :: saveAsFile()
{
	QString filename = KFileDialog::getSaveFileName();
	if(filename.isEmpty()) // abgebrochen oder nichts gewhlt
	{
		return;
	}

	QFile f(filename);
	bool isOK = f.open(IO_WriteOnly);
	if(!isOK)
	{
		KMessageBox::error(this, "Datei konnte nicht geffnet werden", "Dateifehler");
		return;
	}

	QTextStream s(&f); // fr Daten: QDataStream
	edit->saveText(&s); // fr Daten z.B.: s << i << p;
	f.close(); // wrde auch vom Destruktor aufgerufen, aber dann knnten wir nicht schon hier Probleme abfragen
	
	if(f.status() != IO_Ok)
	{
		KMessageBox::error(this, "Fehler beim Schreiben der Datei", "Dateifehler");
	}
}

void KDemoWindow :: insertDate()
{
	QString dateString = QDate::currentDate().toString();
	QTextIStream dateTextIStream(&dateString);
	edit->insertText(&dateTextIStream);
}

void KDemoWindow :: wrap()
{
	KEdit::WordWrap w = edit->wordWrap();
	if(w == KEdit::WidgetWidth)
	{
		edit->setWordWrap(KEdit::NoWrap);
	}
	else
	{
		edit->setWordWrap(KEdit::WidgetWidth);
	}
}

bool KDemoWindow :: queryClose() // Rckgabewert false heit: Wir wollen nicht beenden.
{
	int result = KMessageBox::warningContinueCancel(this, "Der Text geht beim Beenden verloren. Sind Sie sicher?",
		"Sicherheitshalber ...", "Beenden");
	// Besser wre, sich zu merken, ob der Text seit dem letzten Speichern/ffnen verndert wurde,
	// und hier je nach Lage der Dinge anzubieten, ihn am selben Ort zu speichern.

	if(result != KMessageBox::Continue)
	{
		return false;
	}

	KConfig* conf = KApplication::kApplication()->config();
	conf->writeEntry("Wrap", (bool) edit->wordWrap() == KEdit::WidgetWidth);

	return true;
}

void KDemoWindow :: saveProperties(KConfig* conf) // wird vom Session Management aufgerufen
{
	conf->writeEntry("Wrap", (bool) edit->wordWrap() == KEdit::WidgetWidth);
	conf->sync();
}

void KDemoWindow :: readProperties(KConfig* conf) // wird vom Session Management aufgerufen
{
	if(conf->readBoolEntry("Wrap", false))
	{
		edit->setWordWrap(KEdit::WidgetWidth);
		wrapAction->setChecked(true);
	}
	else
	{
		edit->setWordWrap(KEdit::NoWrap);
		wrapAction->setChecked(false);
	}

	// hier sollte man auch den Text wieder einlesen, sich also die Datei gemerkt haben
}

